package
{
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import utility.CUtility;
	import flash.display.BitmapData;
	public class CCreature extends CGameObject
	{
		public var gfx_offset:Point;
		public var shot_offset:Point;
		public var movement:CMovement;
		public var size:int;
		public var attacks:Array;
		public var stats:CStats;
		public var faction:int;
		public function CCreature()
		{
			
		}
		public function init_creature(spd:Number, size:int, stats:CStats, faction:int, gfx_off:Point, shot_off:Point):void
		{
			gfx_offset = gfx_off.clone();
			shot_offset = shot_off.clone();
			movement = new CMovement(spd);
			this.size = size;
			attacks = new Array(4);
			this.stats = stats;
			this.faction = faction;
		}
		override public function update(gm:CGameManager, dt:Number):void
		{
			movement.update(gm.Game.world.currentMap, this, dt);
			if(stats.hp_reg > 0)
				stats.hp = Math.min(stats.maxHp, stats.hp + dt * stats.hp_reg);
		}
		public function passage(gm:CGameManager):void
		{
			var world:CWorld = gm.Game.world;
			if (!world.currentMap.finished) return;
			for (var i:int = 0; i < 4;++i)
			{
				if (world.currentMap.passages[i] && CUtility.Point_Rect_Collide(pos, world.currentMap.passages[i].area))
				{
					var targetId:int = world.currentMap.passages[i].TargetId;
					world.changeMap(gm, world.currentMap.passages[i].targetMapId);
					var p:Point = world.currentMap.setTargetLocation(targetId, world.currentMap);
					gm.Game.dragon.pos.x = p.x;
					gm.Game.dragon.pos.y = p.y;
					gm.Game.dragon.movement.target = null;
					break;
				}
			}
		}
		override public function draw(backbuffer:BitmapData, scroll:Point, gm:CGameManager):void
		{
			if(movement.look_direction==0)
				(ResManager.img[imgId] as AnimImage).draw(backbuffer, pos.x-Math.floor(scroll.x) - gfx_offset.x, pos.y-Math.floor(scroll.y)+ gfx_offset.y, frame);
			else
				(ResManager.img[imgId] as AnimImage).draw(backbuffer, pos.x - Math.floor(scroll.x) + gfx_offset.x, pos.y - Math.floor(scroll.y) + gfx_offset.y, frame + movement.look_direction);
			drawHpBar(backbuffer, scroll, 0);
		}
		
		public function collision(gm:CGameManager, dt:Number):void
		{
			var proj:CProjectile;
			var creature:CCreature;
			movement.evade_direction = null;
			var nextPos:Point = new Point();
			if (movement.Direction)
			{
				nextPos.x = pos.x + dt * movement.speed * movement.Direction.x;
				nextPos.y = pos.y + dt * movement.speed * movement.Direction.y;
			}
			for each (var obj:CGameObject in gm.Game.objectList)
			{
				if (!obj.active)
					continue;
				if (obj is CProjectile)
				{
					proj = obj as CProjectile;
					if (proj.faction != faction && CUtility.inRange(proj.pos, pos, Math.max(proj.collisionRadius+this.collisionRadius, movement.speed * dt * 1.1)) && proj.height <= size)
					{
						if(proj.attackRef)
							this.stats.hit(gm, proj.attackRef.attack, proj.attackRef.caster, proj.attackRef.stats);
						proj.active = false;
						gm.Game.removedGameObjects.addItem(proj);
						proj.execExplosion(gm);
					}
					
				}
				else if (obj is CCreature)
				{
					creature = obj as CCreature;
					var space:int = 0;
					if (this is CEnemy && creature is CEnemy)
						space = 10;
					
					if (creature != this && !movement.evade_direction && movement.target && movement.Direction && CUtility.inRange(nextPos, creature.pos, collisionRadius+creature.collisionRadius+space))
					{
						var angle:Number = Math.atan2(movement.Direction.y, movement.Direction.x);
						movement.evade_direction = new Point();
						if (movement.Direction.x * (creature.pos.y - pos.y) - movement.Direction.y * (creature.pos.x - pos.x) > 0)
						{
							
							movement.evade_direction.x = Math.cos(angle-Math.PI / 2);
							movement.evade_direction.y = Math.sin(angle-Math.PI / 2);
						}
						else
						{
							movement.evade_direction.x = Math.cos(angle+Math.PI / 2);
							movement.evade_direction.y = Math.sin(angle+Math.PI / 2);
						}
					}
				}
			}
		}
		public function drawHpBar(backbuffer:BitmapData, scroll:Point, id:int):void
		{
			var img:AnimImage = ResManager.img[ResManager.IMG_HPBAR];
			img.draw(backbuffer, pos.x - Math.floor(scroll.x), pos.y - Math.floor(scroll.y) + 6, 0 + 2 * id)
			var size:int = 14;
			var offset:int = 9;
			if (id == 1)
			{
				size = 30;
				offset = 1;
			}
			backbuffer.copyPixels(img.getFrame(1 + 2 * id), new Rectangle(offset, 1, size*Math.max(0, Math.min(1, stats.hp/stats.maxHp)), 2), new Point(pos.x - Math.floor(scroll.x)-img.Handle.x+offset, pos.y - Math.floor(scroll.y) + 6-img.Handle.y+1));
		}
	}
}